-- Setting up
StreamBuddy = LibStub("AceAddon-3.0"):NewAddon("StreamBuddy", "AceConsole-3.0")


-- Options available to players
local options = {
	name = "StreamBuddy",
	handler = StreamBuddy,
	type = 'group',
	args = {
		-- A quick call to arms added to the message which the addon will send with the livestream link
		msg = {
			type = "input",
			name = "Message",
			desc = "The message which will be sent with your Livestream's link",
			usage = "<Your message>",
			get = "GetMessage",
			set = "SetMessage",
			width = "full",
		},
		-- Message that will be used when the ad is sent via /emote
		emote = {
			type = "input",
			name = "Emote",
			desc = "The message which will be sent with your Livestream's link via /emote",
			usage = "<Your message>",
			get = "GetEmote",
			set = "SetEmote",
			width = "full",
		},
		-- The link to the user's live stream
		stream = {
			type = "input",
			name = "Channel",
			desc = "The link to your live-streaming channel",
			usage = "<Your livestream channel's link>",
			get = "GetChannel",
			set = "SetChannel",
			width = "full",
		},
		-- Toggles if ad messages will be sent to the party chat
		sendToParty = {
			type = "toggle",
			name = "Send to Party Chat",
			desc = "Should we send an ad message to the party's chat (/p)?",
			get = "IsSentToParty",
			set = "ToggleSendToParty",
		},
		-- Toggles if ad messages will be sent to the raid chat
		sendToRaid = {
			type = "toggle",
			name = "Send to Raid Chat",
			desc = "Should we send an ad message to the raid's chat (/r)?",
			get = "IsSentToRaid",
			set = "ToggleSendToRaid",
		},
		-- Toggles if ad messages will be sent to the instance chat
		sendToInstance = {
			type = "toggle",
			name = "Send to Instance Chat",
			desc = "Should we send an ad message to the instance's chat (/i)?",
			get = "IsSentToInstance",
			set = "ToggleSendToInstance",
		},
		-- Toggles if ad messages will be sent to the raid warning
		sendToRW = {
			type = "toggle",
			name = "Send to Raid Warning Chat",
			desc = "Should we send an ad message via Raid Warning (/rw)?",
			get = "IsSentToRW",
			set = "ToggleSendToRW",
		},
		-- Toggles if ad messages will be sent to the default (/say) chat
		sendToDefault = {
			type = "toggle",
			name = "Send to Default Chat",
			desc = "Should we send an ad message to the default chat (/s)?",
			get = "IsSentToDefault",
			set = "ToggleSendToDefault",
		},
		-- Toggles if ad messages will be sent to the yell (/y) chat
		sendToYell = {
			type = "toggle",
			name = "Send to Yell Chat",
			desc = "Should we send an ad message to the yell chat (/y)?",
			get = "IsSentToYell",
			set = "ToggleSendToYell",
		},
		-- Toggles if ad messages will be sent to the guild (/g) chat
		sendToGuild = {
			type = "toggle",
			name = "Send to Guild Chat",
			desc = "Should we send an ad message to the guild chat (/g)?",
			get = "IsSentToGuild",
			set = "ToggleSendToGuild",
		},
		-- Toggles if the ad message will be set as your DND (/dnd) message
		sendToDND = {
			type = "toggle",
			name = "Send to DND",
			desc = "Should we set the ad message as your DND message (/dnd)?",
			get = "IsSentToDND",
			set = "ToggleSendToDND",
		},
		-- Toggles if the ad message should be set as your Battle.Net message
		sendToBNet = {
			type = "toggle",
			name = "Send to BNet",
			desc = "Should we set the ad message as your BNet message (/BNet)?",
			get = "IsSentToBNet",
			set = "ToggleSendToBNet",
		},
		-- Toggles if the ad message should be sent to the emote (/e or /emote) chat
		sendToEmote = {
			type = "toggle",
			name = "Send to Emote",
			desc = "Should we send the ad message via your Emote message (/Emote)?",
			get = "IsSentToEmote",
			set = "ToggleSendToEmote",
		},
		-- Toggles if "StreamBuddy" will be added to the ad message as a Prefix
		attachSbAd = {
			type = "toggle",
			name = "Attach StreamBuddy Prefix",
			desc = "Should we add the StreamBuddy prefix to the ad message? (<Insert a sad face here if you say no>)",
			get = "ShouldAttachSbAd",
			set = "ToggleAttachSbAd",
		},
		-- ChatCommand, sends the add to the desired chats
		send = {
			type = "input",
			name = "Send Ads",
			desc = "Send stream ads to the selected channels",
			set = "SendComplexAd",
			hidden = true,
		},
	},
}

-- Profiles
local defaults = {
	profile = {
		msg = "Check out my livestream @",
		stream = "https://twitch.tv/arcebus",
		emote = "tells you to check out his livestream @",
		flagSendToParty = false,
		flagSendToDefault = false,
		flagSendToYell = false,
		flagSendToGuild = true,
		flagSendToRaid = false,
		flagSendToInstance = true,
		flagSendToRW = false,
		flagSendToDND = true,
		flagSendToBNet = true,
		flagSendToEmote = true,
		attachSbAdFlag = true
	},
}


-- Addon Methods

-- Called when the addon is initializing
function StreamBuddy:OnInitialize( )
	-- Initializing the database
	self.db = LibStub("AceDB-3.0"):New("StreamBuddyDB", defaults, true)

	-- Initializing the Ace3.0 engine
	LibStub("AceConfig-3.0"):RegisterOptionsTable("StreamBuddy", options, {"streambuddy", "sb"})
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("StreamBuddy", "StreamBuddy")	-- Registering with Blizz's API in order to have options showing up @ the addons tab
	-- Registering the chat commands used by the addon
	self:RegisterChatCommand("streambuddy", "ChatCommand")												
    self:RegisterChatCommand("sb", "ChatCommand")
end

-- Called when the addon is enabled
function StreamBuddy:OnEnable()
    
end

-- Called when the addon is disabled
function StreamBuddy:OnDisable()
    
end

-- Handles /sb and /streambuddy (and sons) chat commands
function StreamBuddy:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0"):HandleCommand("sb", "StreamBuddy", input)
	end
end

-- Handles the /send chat command with args
function StreamBuddy:SendComplexAd(info, newValue)
	-- Checks if it's an empty or nil string
	if not newValue or newValue:trim() == "" then
		-- If it's an empty string, use the user's default message
		newValue = self.db.profile.msg
	end

	-- Builds the ad message
	local adVar = self:PrepareAd(newValue)
	
	-- Sends the chat message to the set chats
	if self:IsSentToParty(nil) then
		SendChatMessage(adVar, "PARTY", nil, nil) -- PARTY
	end

	if self:IsSentToRaid(nil) then
		SendChatMessage(adVar, "RAID", nil, nil) -- RAID
	end

	if self:IsSentToInstance(nil) then
		SendChatMessage(adVar, "INSTANCE_CHAT", nil, nil) -- INSTANCE
	end

	if self:IsSentToRW(nil) then
		SendChatMessage(adVar, "RAID_WARNING", nil, nil) -- RW
	end

	if self:IsSentToDefault(nil) then
		SendChatMessage(adVar, "SAY", nil, nil) -- DEFAULT
	end

	if self:IsSentToGuild(nil) then
		SendChatMessage(adVar, "GUILD", nil, nil) -- GUILD
	end

	if self:IsSentToYell(nil) then
		SendChatMessage(adVar, "YELL", nil, nil) -- YELL
	end
	
	if self:IsSentToDND(nil) then
		SendChatMessage(adVar, "DND", nil, nil) -- DND
	end

	if self:IsSentToEmote(nil) then
		local adEmoteVar = self:EmoteAd(nil)
		SendChatMessage(adEmoteVar, "EMOTE", nil, nil) -- Emote
	end

	isConnectedBNet = BNConnected()	-- Checks if the player is actually connected to Battle.net
	if self:IsSentToBNet(nil) and isConnectedBNet then
		BNSetCustomMessage(adVar) -- Battle.Net
	end

	self:Print("Sending your ad messages! :D")
end

-- Prepares the ad which will be sent
function StreamBuddy:PrepareAd(message)
	if self:ShouldAttachSbAd(nil) then
		return ("StreamBuddy: " .. message .. " " .. self.db.profile.stream)
	else
		return (message .. " " .. self.db.profile.stream)
	end
	
end

-- Prepares the ad which will be sent
function StreamBuddy:EmoteAd(info)
	return (self:GetEmote(nil) .. " " .. self.db.profile.stream)
	
end

-- Options Getters and Setters

-- Gets the current stream
function StreamBuddy:GetChannel(info)
    return self.db.profile.stream
end

-- Sets a new stream
function StreamBuddy:SetChannel(info, newValue)
    self.db.profile.stream = newValue
end

-- Gets the current message
function StreamBuddy:GetMessage(info)
	return self.db.profile.msg
end

-- Sets a new message
function StreamBuddy:SetMessage(info, newValue)
	self.db.profile.msg = newValue
end

-- Gets the current Emote
function StreamBuddy:GetEmote(info)
	return self.db.profile.emote
end

-- Sets a new Emote
function StreamBuddy:SetEmote(info, newValue)
	self.db.profile.emote = newValue
end

-- Gets the current sendToParty status
function StreamBuddy:IsSentToParty(info)
	return self.db.profile.flagSendToParty
end

-- Sets a new sendToParty status
function StreamBuddy:ToggleSendToParty(info, newValue)
	self.db.profile.flagSendToParty = newValue
end

-- Gets the current sendToDefault status
function StreamBuddy:IsSentToDefault(info)
	return self.db.profile.flagSendToDefault
end

-- Sets a new sendToDefault status
function StreamBuddy:ToggleSendToDefault(info, newValue)
	self.db.profile.flagSendToDefault = newValue
end

-- Gets the current sendToYell status
function StreamBuddy:IsSentToYell(info)
	return self.db.profile.flagSendToYell
end

-- Sets a new sendToYell status
function StreamBuddy:ToggleSendToYell(info, newValue)
	self.db.profile.flagSendToYell = newValue
end

-- Gets the current sendToGuild status
function StreamBuddy:IsSentToGuild(info)
	return self.db.profile.flagSendToGuild
end

-- Sets a new sendToGuild status
function StreamBuddy:ToggleSendToGuild(info, newValue)
	self.db.profile.flagSendToGuild = newValue
end

-- Gets the current sendToRaid status
function StreamBuddy:IsSentToRaid(info)
	return self.db.profile.flagSendToRaid
end

-- Sets a new sendToRaid status
function StreamBuddy:ToggleSendToRaid(info, newValue)
	self.db.profile.flagSendToRaid = newValue
end

-- Gets the current sendToInstance status
function StreamBuddy:IsSentToInstance(info)
	return self.db.profile.flagSendToInstance
end

-- Sets a new sendToInstance status
function StreamBuddy:ToggleSendToInstance(info, newValue)
	self.db.profile.flagSendToInstance = newValue
end

-- Gets the current sendToRW status
function StreamBuddy:IsSentToRW(info)
	return self.db.profile.flagSendToRW
end

-- Sets a new sendToRW status
function StreamBuddy:ToggleSendToRW(info, newValue)
	self.db.profile.flagSendToRW = newValue
end

-- Gets the current sendToDND status
function StreamBuddy:IsSentToDND(info)
	return self.db.profile.flagSendToDND
end

-- Sets a new sendToDND status
function StreamBuddy:ToggleSendToDND(info, newValue)
	self.db.profile.flagSendToDND = newValue
end

-- Gets the current sendToBNet status
function StreamBuddy:IsSentToBNet(info)
	return self.db.profile.flagSendToBNet
end

-- Sets a new sendToBNet status
function StreamBuddy:ToggleSendToBNet(info, newValue)
	self.db.profile.flagSendToBNet = newValue
end

-- Gets the current sendToBNet status
function StreamBuddy:ShouldAttachSbAd(info)
	return self.db.profile.attachSbAdFlag
end

-- Sets a new sendToBNet status
function StreamBuddy:ToggleAttachSbAd(info, newValue)
	self.db.profile.attachSbAdFlag = newValue
end

-- Gets the current sendToEmote
function StreamBuddy:IsSentToEmote(info)
	return self.db.profile.flagSendToEmote
end

-- Sets a new sendToEmote
function StreamBuddy:ToggleSendToEmote(info, newValue)
	self.db.profile.flagSendToEmote = newValue
end